"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BlockchainAction = void 0;
const core_1 = require("../../core/index.js");
const instructions_1 = require("../../instructions/index.js");
const borrowObligationLiquidity_solana_1 = require("../../instructions/borrowObligationLiquidity.solana.js");
const depositObligationCollateral_solana_1 = require("../../instructions/depositObligationCollateral.solana.js");
const depositReserveLiquidity_solana_1 = require("../../instructions/depositReserveLiquidity.solana.js");
const redeemReserveCollateral_solana_1 = require("../../instructions/redeemReserveCollateral.solana.js");
const web3_js_1 = require("@solana/web3.js");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const bn_js_1 = __importDefault(require("bn.js"));
const constants_1 = require("../constants.js");
const shared_1 = require("../shared.js");
class BlockchainAction {
    network;
    programId;
    connection;
    reserve;
    lendingMarket;
    publicKey;
    obligationAddress;
    obligationAccountInfo;
    userTokenAccountAddress;
    userCollateralAccountAddress;
    seed;
    symbol;
    positions;
    amount;
    hostAta;
    setupIxs;
    lendingIxs;
    cleanupIxs;
    preTxnIxs;
    postTxnIxs;
    depositReserves;
    borrowReserves;
    supplyToEarnApys;
    supplyToEarnRewards;
    nftVoucherAddress;
    nftAtaAddress;
    nftInterestAmount;
    steInstruction;
    constructor(options) {
        const { network = shared_1.BlockchainNetwork.RENEC, programId, connection, reserve, lendingMarket, publicKey, obligationAddress, obligationAccountInfo, userTokenAccountAddress, userCollateralAccountAddress, seed, symbol, positions, amount, depositReserves, borrowReserves, hostAta, setupIxs = [], supplyToEarnApys = [], supplyToEarnRewards = [], nftVoucherAddress, nftAtaAddress, nftInterestAmount, } = options;
        this.network = network;
        this.programId = programId;
        this.connection = connection;
        this.publicKey = publicKey;
        this.amount = new bn_js_1.default(amount);
        this.symbol = symbol;
        this.positions = positions;
        this.hostAta = hostAta;
        this.obligationAccountInfo = obligationAccountInfo;
        this.lendingMarket = lendingMarket;
        this.seed = seed;
        this.reserve = reserve;
        this.obligationAddress = obligationAddress;
        this.userTokenAccountAddress = userTokenAccountAddress;
        this.userCollateralAccountAddress = userCollateralAccountAddress;
        this.setupIxs = setupIxs;
        this.lendingIxs = [];
        this.cleanupIxs = [];
        this.preTxnIxs = [];
        this.postTxnIxs = [];
        this.depositReserves = depositReserves;
        this.borrowReserves = borrowReserves;
        this.supplyToEarnApys = supplyToEarnApys;
        this.supplyToEarnRewards = supplyToEarnRewards;
        this.nftVoucherAddress = nftVoucherAddress;
        this.nftAtaAddress = nftAtaAddress;
        this.nftInterestAmount = nftInterestAmount ? new bn_js_1.default(nftInterestAmount) : new bn_js_1.default(0);
    }
    // Concrete method: can be inherited and used by subclasses directly
    async getTransactions() {
        const txns = {
            preLendingTxn: null,
            lendingTxn: null,
            postLendingTxn: null,
        };
        if (this.preTxnIxs.length) {
            txns.preLendingTxn = new web3_js_1.Transaction({
                feePayer: this.publicKey,
                recentBlockhash: (await this.connection.getRecentBlockhash()).blockhash,
            }).add(...this.preTxnIxs);
        }
        txns.lendingTxn = new web3_js_1.Transaction({
            feePayer: this.publicKey,
            recentBlockhash: (await this.connection.getRecentBlockhash()).blockhash,
        }).add(...this.setupIxs, ...this.lendingIxs, ...this.cleanupIxs);
        if (this.postTxnIxs.length) {
            txns.postLendingTxn = new web3_js_1.Transaction({
                feePayer: this.publicKey,
                recentBlockhash: (await this.connection.getRecentBlockhash()).blockhash,
            }).add(...this.postTxnIxs);
        }
        return txns;
    }
    async sendSingleTransaction(txn, sendTransaction, callback) {
        if (!txn)
            return "";
        const signature = await sendTransaction(txn, this.connection);
        if (callback) {
            callback();
        }
        await this.connection.confirmTransaction(signature);
        return signature;
    }
    async sendTransactions(sendTransaction, preCallback, lendingCallback, postCallback) {
        const txns = await this.getTransactions();
        await this.sendSingleTransaction(txns.preLendingTxn, sendTransaction, preCallback);
        const signature = await this.sendSingleTransaction(txns.lendingTxn, sendTransaction, lendingCallback);
        await this.sendSingleTransaction(txns.postLendingTxn, sendTransaction, postCallback);
        return signature;
    }
    addDepositReserveLiquidityIx() {
        const commonParams = [
            this.amount,
            this.userTokenAccountAddress,
            this.userCollateralAccountAddress,
            new web3_js_1.PublicKey(this.reserve.address),
            new web3_js_1.PublicKey(this.reserve.liquidityAddress),
            new web3_js_1.PublicKey(this.reserve.collateralMintAddress),
            new web3_js_1.PublicKey(this.lendingMarket.address),
            new web3_js_1.PublicKey(this.lendingMarket.authorityAddress),
            this.publicKey,
            this.programId,
        ];
        if (this.network === shared_1.BlockchainNetwork.RENEC) {
            this.lendingIxs.push((0, instructions_1.depositReserveLiquidityRenecInstruction)(...commonParams));
        }
        else {
            this.lendingIxs.push((0, depositReserveLiquidity_solana_1.depositReserveLiquiditySolanaInstruction)(...commonParams));
        }
    }
    addRedeemReserveCollateralIx() {
        const commonParams = [
            this.amount,
            this.userCollateralAccountAddress,
            this.userTokenAccountAddress,
            new web3_js_1.PublicKey(this.reserve.address),
            new web3_js_1.PublicKey(this.reserve.collateralMintAddress),
            new web3_js_1.PublicKey(this.reserve.liquidityAddress),
            new web3_js_1.PublicKey(this.lendingMarket.address),
            new web3_js_1.PublicKey(this.lendingMarket.authorityAddress),
            this.publicKey,
            this.programId,
        ];
        if (this.network === shared_1.BlockchainNetwork.RENEC) {
            this.lendingIxs.push((0, instructions_1.redeemReserveCollateralRenecInstruction)(...commonParams));
        }
        else {
            this.lendingIxs.push((0, redeemReserveCollateral_solana_1.redeemReserveCollateralSolanaInstruction)(...commonParams));
        }
    }
    addDepositObligationCollateralIx() {
        const commonParams = [
            this.amount,
            this.userCollateralAccountAddress,
            new web3_js_1.PublicKey(this.reserve.collateralSupplyAddress),
            new web3_js_1.PublicKey(this.reserve.address),
            this.obligationAddress,
            new web3_js_1.PublicKey(this.lendingMarket.address),
            this.publicKey,
            this.publicKey,
            this.programId,
        ];
        if (this.network === shared_1.BlockchainNetwork.RENEC) {
            this.lendingIxs.push((0, instructions_1.depositObligationCollateralRenecInstruction)(...commonParams));
        }
        else {
            this.lendingIxs.push((0, depositObligationCollateral_solana_1.depositObligationCollateralSolanaInstruction)(...commonParams));
        }
    }
    addBorrowIx() {
        const commonParams = [
            this.amount,
            new web3_js_1.PublicKey(this.reserve.liquidityAddress),
            this.userTokenAccountAddress,
            new web3_js_1.PublicKey(this.reserve.address),
            new web3_js_1.PublicKey(this.reserve.liquidityFeeReceiverAddress),
            this.obligationAddress,
            new web3_js_1.PublicKey(this.lendingMarket.address),
            new web3_js_1.PublicKey(this.lendingMarket.authorityAddress),
            this.publicKey,
            this.programId,
        ];
        if (this.network === shared_1.BlockchainNetwork.RENEC) {
            this.lendingIxs.push((0, instructions_1.borrowObligationLiquidityRenecInstruction)(...commonParams, this.hostAta));
        }
        else {
            this.lendingIxs.push((0, borrowObligationLiquidity_solana_1.borrowObligationLiquiditySolanaInstruction)(...commonParams, this.depositReserves, this.hostAta));
        }
    }
    formatReserveData(parsedData) {
        const totalBorrowsWads = parsedData.liquidity.borrowedAmountWads;
        const totalLiquidityWads = parsedData.liquidity.availableAmount.mul(new bn_js_1.default(constants_1.WAD));
        const totalDepositsWads = totalBorrowsWads.add(totalLiquidityWads);
        const cTokenExchangeRate = new bignumber_js_1.default(totalDepositsWads.toString())
            .div(parsedData.collateral.mintTotalSupply.toString())
            .div(constants_1.WAD)
            .toNumber();
        return {
            // Reserve config
            optimalUtilizationRate: parsedData.config.optimalUtilizationRate / 100,
            loanToValueRatio: parsedData.config.loanToValueRatio / 100,
            liquidationBonus: parsedData.config.liquidationBonus / 100,
            liquidationThreshold: parsedData.config.liquidationThreshold / 100,
            minBorrowRate: parsedData.config.minBorrowRate / 100,
            optimalBorrowRate: parsedData.config.optimalBorrowRate / 100,
            maxBorrowRate: parsedData.config.maxBorrowRate / 100,
            protocolTakeRate: parsedData.config.protocolTakeRate / 100,
            borrowFeePercentage: new bignumber_js_1.default(parsedData.config.fees.borrowFeeWad.toString()).dividedBy(constants_1.WAD).toNumber(),
            hostFeePercentage: parsedData.config.fees.hostFeePercentage / 100,
            flashLoanFeePercentage: new bignumber_js_1.default(parsedData.config.fees.flashLoanFeeWad.toString())
                .dividedBy(constants_1.WAD)
                .toNumber(),
            depositLimit: parsedData.config.depositLimit,
            reserveBorrowLimit: parsedData.config.borrowLimit,
            // Reserve info
            name: this.reserve.liquidityToken.name,
            symbol: this.reserve.liquidityToken.symbol,
            decimals: this.reserve.liquidityToken.decimals,
            mintAddress: this.reserve.liquidityToken.mint,
            totalDepositsWads,
            totalBorrowsWads,
            totalLiquidityWads,
            supplyInterestAPY: this.calculateSupplyAPY(parsedData),
            borrowInterestAPY: this.calculateBorrowAPY(parsedData),
            assetPriceUSD: new bignumber_js_1.default(parsedData.liquidity.marketPrice.toString()).div(constants_1.WAD).toNumber(),
            userDepositLimit: this.reserve.userSupplyCap,
            cumulativeBorrowRateWads: parsedData.liquidity.cumulativeBorrowRateWads,
            cTokenExchangeRate,
        };
    }
    calculateSupplyAPY = (reserve) => {
        const apr = this.calculateSupplyAPR(reserve);
        const apy = new bignumber_js_1.default(1).plus(new bignumber_js_1.default(apr).dividedBy(core_1.SLOTS_PER_YEAR)).toNumber() ** core_1.SLOTS_PER_YEAR - 1;
        return apy;
    };
    calculateBorrowAPY = (reserve) => {
        const apr = this.calculateBorrowAPR(reserve);
        const apy = new bignumber_js_1.default(1).plus(new bignumber_js_1.default(apr).dividedBy(core_1.SLOTS_PER_YEAR)).toNumber() ** core_1.SLOTS_PER_YEAR - 1;
        return apy;
    };
    calculateSupplyAPR(reserve) {
        const currentUtilization = this.calculateUtilizationRatio(reserve);
        const borrowAPY = this.calculateBorrowAPR(reserve);
        return currentUtilization.multipliedBy(borrowAPY);
    }
    calculateUtilizationRatio(reserve) {
        const totalBorrowsWads = new bignumber_js_1.default(reserve.liquidity.borrowedAmountWads.toString()).div(constants_1.WAD);
        const currentUtilization = totalBorrowsWads.dividedBy(totalBorrowsWads.plus(reserve.liquidity.availableAmount.toString()));
        return currentUtilization;
    }
    calculateBorrowAPR(reserve) {
        const currentUtilization = this.calculateUtilizationRatio(reserve);
        const optimalUtilization = reserve.config.optimalUtilizationRate / 100;
        let borrowAPR;
        if (optimalUtilization === 1.0 || currentUtilization.isLessThan(optimalUtilization)) {
            const normalizedFactor = currentUtilization.div(optimalUtilization);
            const optimalBorrowRate = reserve.config.optimalBorrowRate / 100;
            const minBorrowRate = reserve.config.minBorrowRate / 100;
            borrowAPR = normalizedFactor.multipliedBy(optimalBorrowRate - minBorrowRate).plus(minBorrowRate);
        }
        else {
            const normalizedFactor = currentUtilization.minus(optimalUtilization).div(1 - optimalUtilization);
            const optimalBorrowRate = reserve.config.optimalBorrowRate / 100;
            const maxBorrowRate = reserve.config.maxBorrowRate / 100;
            borrowAPR = normalizedFactor.multipliedBy(maxBorrowRate - optimalBorrowRate).plus(optimalBorrowRate);
        }
        return borrowAPR;
    }
}
exports.BlockchainAction = BlockchainAction;
