"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RelendAction = void 0;
const constants_helper_1 = require("../../core/constants.helper.js");
const supply_to_earn_1 = require("../../supply-to-earn/index.js");
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const bn_js_1 = __importDefault(require("bn.js"));
const constants_common_1 = require("../../core/constants.common.js");
const constants_renec_1 = require("../../core/constants.renec.js");
const instructions_1 = require("../../instructions/index.js");
const obligation_1 = require("../../state/obligation.js");
const reserve_1 = require("../../state/reserve.js");
const constants_1 = require("../constants.js");
const utils_1 = require("../utils.js");
const blockchain_action_abstract_1 = require("./blockchain_action.abstract.js");
const RENEC_PADDING_FOR_INTEREST = "1000000";
function getTokenInfo(symbol, solendInfo) {
    const tokenInfo = solendInfo.reserves.find((reserve) => reserve.liquidityToken.symbol === symbol);
    if (!tokenInfo) {
        throw new Error(`Could not find ${symbol} in ASSETS`);
    }
    return tokenInfo;
}
const PRICE_PAIRS = (0, constants_renec_1.getRenecPricePair)();
const REUSD_PRICE = (0, constants_renec_1.getRenecReUSDPricePair)();
const REVND_PRICE = constants_renec_1.RENEC_PRICE_PAIRS_MAINNET_TEST_REVND_RENGN[0];
const RENGN_PRICE = constants_renec_1.RENEC_PRICE_PAIRS_MAINNET_TEST_REVND_RENGN[1];
const WITHDRAW_BUFFER_SLOTS = 100;
const WITHDRAW_MAX_ERROR_PERCENTAGE = 0.0001;
class RelendAction extends blockchain_action_abstract_1.BlockchainAction {
    static async initialize(data) {
        const { network, action, amount, symbol, publicKey, connection, supplyToEarnApys, supplyToEarnRewards, lendingMarketAddress, hostAddress, nftVoucherAddress, nftInterestAmount, } = data;
        const relendInfo = constants_helper_1.ConstantsHelper.getConfig(network);
        let lendingMarket;
        if (lendingMarketAddress) {
            lendingMarket = relendInfo.find((market) => market.address == lendingMarketAddress.toBase58());
            if (!lendingMarket) {
                throw `market address not found: ${lendingMarketAddress}`;
            }
        }
        else {
            lendingMarket = relendInfo.find((market) => market.isPrimary) ?? relendInfo[0];
        }
        const seed = lendingMarket.address.slice(0, 32);
        const programId = constants_helper_1.ConstantsHelper.getProgramId(network);
        const obligationAddress = await web3_js_1.PublicKey.createWithSeed(publicKey, seed, programId);
        const reserve = lendingMarket.reserves.find((res) => res.liquidityToken.symbol === symbol);
        if (!reserve) {
            throw new Error(`Could not find asset ${symbol} in reserves`);
        }
        const obligationAccountInfo = await connection.getAccountInfo(obligationAddress);
        let obligationDetails = null;
        const depositReserves = [];
        const borrowReserves = [];
        if (obligationAccountInfo) {
            obligationDetails = (0, obligation_1.parseObligation)(web3_js_1.PublicKey.default, obligationAccountInfo).info;
            obligationDetails.deposits.forEach((deposit) => {
                depositReserves.push(deposit.depositReserve);
            });
            obligationDetails.borrows.forEach((borrow) => {
                borrowReserves.push(borrow.borrowReserve);
            });
        }
        // Union of addresses
        const distinctReserveCount = [
            ...Array.from(new Set([...borrowReserves.map((e) => e.toBase58()), ...(action === "borrow" ? [reserve.address] : [])])),
        ].length +
            [
                ...Array.from(new Set([...depositReserves.map((e) => e.toBase58()), ...(action === "deposit" ? [reserve.address] : [])])),
            ].length;
        if (distinctReserveCount > constants_common_1.POSITION_LIMIT) {
            throw Error(`Obligation already has max number of positions: ${constants_common_1.POSITION_LIMIT}`);
        }
        const tokenInfo = getTokenInfo(symbol, lendingMarket);
        const userTokenAccountAddress = await spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, new web3_js_1.PublicKey(tokenInfo.liquidityToken.mint), publicKey, true);
        const userCollateralAccountAddress = await spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, new web3_js_1.PublicKey(reserve.collateralMintAddress), publicKey, true);
        // Host ATA
        let hostAta = undefined;
        const setupIxs = [];
        if (hostAddress) {
            const hostTokenAccountAddress = await spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, new web3_js_1.PublicKey(reserve.liquidityToken.mint), new web3_js_1.PublicKey(hostAddress));
            const hostTokenAccountInfo = await connection.getAccountInfo(hostTokenAccountAddress);
            if (!hostTokenAccountInfo) {
                setupIxs.push((0, utils_1.createAssociatedTokenAccountInstruction)(publicKey, hostTokenAccountAddress, hostAddress, new web3_js_1.PublicKey(reserve.liquidityToken.mint)));
            }
            hostAta = hostTokenAccountAddress;
        }
        // nft ATA
        let nftAtaAddress = undefined;
        if (nftVoucherAddress) {
            nftAtaAddress = await spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, nftVoucherAddress, publicKey);
            const nftAssociatedTokenAccountInfo = await connection.getAccountInfo(nftAtaAddress);
            if (!nftAssociatedTokenAccountInfo) {
                setupIxs.push((0, utils_1.createAssociatedTokenAccountInstruction)(nftAtaAddress, publicKey, publicKey, nftVoucherAddress));
            }
        }
        return new RelendAction({
            network,
            programId,
            connection,
            reserve,
            lendingMarket,
            publicKey,
            obligationAddress,
            obligationAccountInfo: obligationDetails,
            userTokenAccountAddress,
            userCollateralAccountAddress,
            seed,
            symbol,
            positions: distinctReserveCount,
            amount,
            depositReserves,
            borrowReserves,
            hostAta,
            setupIxs,
            supplyToEarnApys,
            supplyToEarnRewards,
            nftVoucherAddress,
            nftAtaAddress,
            nftInterestAmount,
        });
    }
    static async buildDepositTxns({ network, connection, amount, symbol, publicKey, supplyToEarnApys, supplyToEarnRewards, lendingMarketAddress, }) {
        const axn = await RelendAction.initialize({
            network,
            action: "deposit",
            amount,
            symbol,
            publicKey,
            connection,
            supplyToEarnApys,
            supplyToEarnRewards,
            lendingMarketAddress,
        });
        await axn.addSupportIxs("deposit");
        await axn.addDepositIx();
        return axn;
    }
    static async buildBorrowTxns({ network, connection, amount, symbol, publicKey, supplyToEarnApys, supplyToEarnRewards, lendingMarketAddress, hostAddress, }) {
        const axn = await RelendAction.initialize({
            network,
            action: "borrow",
            amount,
            symbol,
            publicKey,
            connection,
            supplyToEarnApys,
            supplyToEarnRewards,
            lendingMarketAddress,
            hostAddress,
        });
        await axn.addSupportIxs("borrow");
        axn.addBorrowIx();
        return axn;
    }
    static async buildDepositReserveLiquidityTxns({ network, connection, amount, symbol, publicKey, supplyToEarnApys, supplyToEarnRewards, lendingMarketAddress, }) {
        const axn = await RelendAction.initialize({
            network,
            action: "mint",
            amount,
            symbol,
            publicKey,
            connection,
            supplyToEarnApys,
            supplyToEarnRewards,
            lendingMarketAddress,
        });
        await axn.addSupportIxs("mint");
        axn.addDepositReserveLiquidityIx();
        return axn;
    }
    static async buildRedeemReserveCollateralTxns({ network, connection, amount, symbol, publicKey, supplyToEarnApys, supplyToEarnRewards, lendingMarketAddress, }) {
        const axn = await RelendAction.initialize({
            network,
            action: "redeem",
            amount,
            symbol,
            publicKey,
            connection,
            supplyToEarnApys,
            supplyToEarnRewards,
            lendingMarketAddress,
        });
        await axn.addSupportIxs("redeem");
        axn.addRedeemReserveCollateralIx();
        return axn;
    }
    static async buildDepositObligationCollateralTxns({ network, connection, amount, symbol, publicKey, supplyToEarnApys, supplyToEarnRewards, lendingMarketAddress, }) {
        const axn = await RelendAction.initialize({
            network,
            action: "depositCollateral",
            amount,
            symbol,
            publicKey,
            connection,
            supplyToEarnApys,
            supplyToEarnRewards,
            lendingMarketAddress,
        });
        await axn.addSupportIxs("depositCollateral");
        axn.addDepositObligationCollateralIx();
        return axn;
    }
    static async buildWithdrawTxns({ network, connection, amount, symbol, publicKey, supplyToEarnApys, supplyToEarnRewards, lendingMarketAddress, }) {
        const axn = await RelendAction.initialize({
            network,
            action: "withdraw",
            amount,
            symbol,
            publicKey,
            connection,
            supplyToEarnApys,
            supplyToEarnRewards,
            lendingMarketAddress,
        });
        await axn.addSupportIxs("withdraw");
        await axn.addWithdrawIx();
        return axn;
    }
    static async buildRepayTxns({ network, connection, amount, symbol, publicKey, supplyToEarnApys, supplyToEarnRewards, lendingMarketAddress, nftVoucherAddress, nftInterestAmount, }) {
        const axn = await RelendAction.initialize({
            network,
            action: "repay",
            amount,
            symbol,
            publicKey,
            connection,
            supplyToEarnApys,
            supplyToEarnRewards,
            lendingMarketAddress,
            nftVoucherAddress,
            nftInterestAmount,
        });
        await axn.addSupportIxs("repay");
        if (nftVoucherAddress)
            await axn.addRepayNFTVoucherIx();
        else
            await axn.addRepayIx();
        return axn;
    }
    async addDepositIx() {
        const reservePublicKey = new web3_js_1.PublicKey(this.reserve.address);
        const steRewardMatch = this.supplyToEarnApys.find((ste) => ste.account.reserve === this.reserve.address);
        const reserveDeposited = this.obligationAccountInfo?.deposits.find((d) => d.depositReserve.toBase58() === this.reserve.address);
        const hasDepositedAmount = reserveDeposited?.depositedAmount.gt(new bn_js_1.default(0));
        this.steInstruction = await (0, supply_to_earn_1.supplyToEarnInstruction)(this.connection, reservePublicKey, new web3_js_1.PublicKey(this.obligationAddress), this.publicKey);
        const currentTime = Math.floor(Date.now() / 1000);
        const isInEventTime = steRewardMatch && steRewardMatch.account.startTime < currentTime && currentTime < steRewardMatch.account.endTime;
        if (steRewardMatch && !hasDepositedAmount && steRewardMatch.account.apy > 0 && isInEventTime) {
            this.addSteInstructionWhenDepositing();
        }
        // lending ix
        this.lendingIxs.push((0, instructions_1.depositReserveLiquidityAndObligationCollateralInstruction)(this.amount, this.userTokenAccountAddress, this.userCollateralAccountAddress, reservePublicKey, new web3_js_1.PublicKey(this.reserve.liquidityAddress), new web3_js_1.PublicKey(this.reserve.collateralMintAddress), new web3_js_1.PublicKey(this.lendingMarket.address), new web3_js_1.PublicKey(this.lendingMarket.authorityAddress), new web3_js_1.PublicKey(this.reserve.collateralSupplyAddress), // destinationCollateral
        this.obligationAddress, // obligation
        this.publicKey, // obligationOwner
        new web3_js_1.PublicKey(this.reserve.pythOracle), new web3_js_1.PublicKey(this.reserve.switchboardOracle), this.publicKey, // transferAuthority
        this.programId));
        if (steRewardMatch && hasDepositedAmount && steRewardMatch.account.apy > 0) {
            this.addSteInstructionWhenDepositing();
        }
    }
    async addWithdrawIx() {
        const buffer = await this.connection.getAccountInfo(new web3_js_1.PublicKey(this.reserve.address), "processed");
        if (!buffer) {
            throw Error(`Unable to fetch reserve data for ${this.reserve.liquidityToken.name}`);
        }
        const parsedData = (0, reserve_1.parseReserve)(new web3_js_1.PublicKey(this.reserve.address), buffer)?.info;
        if (!parsedData) {
            throw Error(`Unable to parse data of reserve ${this.reserve.liquidityToken.name}`);
        }
        if (!this.obligationAccountInfo) {
            throw Error(`No Obligation Account Info`);
        }
        // Accure Interest
        const selectedObligation = this.obligationAccountInfo.deposits.find((o) => o.depositReserve.toBase58() === this.reserve.address);
        const reserveData = this.formatReserveData(parsedData);
        const currentBorrowRate = this.calculateBorrowAPR(parsedData);
        const currentSlot = await this.connection.getSlot();
        const slotElapsed = currentSlot - (parsedData.lastUpdate.slot.toNumber() ?? 0) + WITHDRAW_BUFFER_SLOTS;
        const compoundedInterestRateWhenRefreshReserve = (0, utils_1.calculateCompoundedInterestRate)(currentBorrowRate, slotElapsed);
        const reserveBorrowedAmount = new bignumber_js_1.default(parsedData.liquidity.borrowedAmountWads.toString()).div(constants_1.WAD);
        const netNewDebt = reserveBorrowedAmount
            .multipliedBy(compoundedInterestRateWhenRefreshReserve)
            .minus(reserveBorrowedAmount);
        const newBorrowedAmountAfterRefresh = reserveBorrowedAmount.plus(netNewDebt);
        const accumulatedProtocolFees = new bignumber_js_1.default(parsedData.liquidity.accumulatedProtocolFeesWads.toString()).div(constants_1.WAD);
        const newAccumulatedProtocolTakeFee = netNewDebt
            .multipliedBy(reserveData.protocolTakeRate / 100)
            .plus(accumulatedProtocolFees);
        const availableAmount = new bignumber_js_1.default(parsedData.liquidity.availableAmount.toString());
        const totalLiquidity = availableAmount.plus(newBorrowedAmountAfterRefresh).minus(newAccumulatedProtocolTakeFee);
        const mintTotalSupply = parsedData.collateral.mintTotalSupply;
        const cTokenExchangeRate = new bignumber_js_1.default(totalLiquidity.div(new bignumber_js_1.default(mintTotalSupply.toString())));
        let cTokenAmountToSubmit = this.amount.eq(new bn_js_1.default(constants_1.U64_MAX))
            ? this.amount
            : new bn_js_1.default(new bignumber_js_1.default(this.amount.toString())
                .dividedBy(cTokenExchangeRate)
                .integerValue(bignumber_js_1.default.ROUND_HALF_FLOOR)
                .toString());
        const DEFAULT_AMOUNT = "1";
        const cTokenToSubmitInBigNumber = new bignumber_js_1.default(cTokenAmountToSubmit.toString());
        const depositedAmountInBigNumber = new bignumber_js_1.default(selectedObligation?.depositedAmount.toString() ?? DEFAULT_AMOUNT);
        if (new bignumber_js_1.default(1)
            .minus(cTokenToSubmitInBigNumber.div(depositedAmountInBigNumber))
            .isLessThanOrEqualTo(WITHDRAW_MAX_ERROR_PERCENTAGE)) {
            cTokenAmountToSubmit = selectedObligation?.depositedAmount ?? new bn_js_1.default(DEFAULT_AMOUNT);
        }
        // Supply to earn ix then lending ix
        const steRewardMatch = this.supplyToEarnRewards.find((ste) => ste.reserve === this.reserve.address);
        if (steRewardMatch && Number(steRewardMatch.totalRewardNumber) !== 0) {
            const steInstruction = await (0, supply_to_earn_1.claimSupplyToEarnRewardInstruction)(this.connection, new web3_js_1.PublicKey(this.reserve.address), new web3_js_1.PublicKey(this.obligationAddress), this.publicKey, steRewardMatch.address);
            /**
             * - with RENEC, we need to submit the steInstruction before the withdraw instruction the tx may fail with error Transaction too large: 1234 > 1232
             * - when positions > 2, we also need to submit the steInstruction before the withdraw instruction the tx may fail with error Transaction too large: 1234 > 1232
             * Ref: https://remitano.slack.com/archives/C066Z4WV83G/p1717986581609539
             */
            const isNotRENEC = this.symbol !== constants_common_1.RENEC;
            const MAX_POSITIONS_WITH_STE = 2;
            const hasManyPositions = steInstruction.length <= 1 && (this.positions ?? 0) <= MAX_POSITIONS_WITH_STE;
            if (hasManyPositions && isNotRENEC) {
                this.lendingIxs.push(...steInstruction);
            }
            else {
                this.preTxnIxs.push(...steInstruction);
            }
        }
        this.lendingIxs.push((0, instructions_1.withdrawObligationCollateralAndRedeemReserveRenecLiquidity)(cTokenAmountToSubmit, new web3_js_1.PublicKey(this.reserve.collateralSupplyAddress), this.userCollateralAccountAddress, new web3_js_1.PublicKey(this.reserve.address), this.obligationAddress, new web3_js_1.PublicKey(this.lendingMarket.address), new web3_js_1.PublicKey(this.lendingMarket.authorityAddress), this.userTokenAccountAddress, // destinationLiquidity
        new web3_js_1.PublicKey(this.reserve.collateralMintAddress), new web3_js_1.PublicKey(this.reserve.liquidityAddress), this.publicKey, // obligationOwner
        this.publicKey, // transferAuthority
        this.programId));
    }
    async addRepayIx() {
        this.lendingIxs.push((0, instructions_1.repayObligationLiquidityInstruction)(this.amount, this.userTokenAccountAddress, new web3_js_1.PublicKey(this.reserve.liquidityAddress), new web3_js_1.PublicKey(this.reserve.address), this.obligationAddress, new web3_js_1.PublicKey(this.lendingMarket.address), this.publicKey, this.programId));
    }
    async addRepayNFTVoucherIx() {
        if (!this.nftVoucherAddress || !this.nftAtaAddress)
            return;
        const reserveAddress = this.reserve.address;
        const pda = (0, constants_renec_1.getRenecPDAOracle)();
        const reserveInfo = this.lendingMarket.reserves.find((reserve) => reserve.address === reserveAddress);
        if (!reserveInfo) {
            throw new Error(`Could not find asset ${reserveAddress} in reserves`);
        }
        const symbol = reserveInfo.liquidityToken.symbol;
        let pair;
        if (symbol === constants_common_1.RE_USD) {
            pair = REUSD_PRICE;
        }
        else if (symbol === constants_common_1.RE_VND) {
            pair = REVND_PRICE;
        }
        else if (symbol === constants_common_1.RE_NGN) {
            pair = RENGN_PRICE;
        }
        else {
            pair = PRICE_PAIRS.find((pair) => pair.baseSymbol === reserveInfo.liquidityToken.symbol);
        }
        const oracleProduct = pda.product(new web3_js_1.PublicKey(pair.quoteToken), new web3_js_1.PublicKey(pair.baseToken));
        console.log("this.reserve.address", this.reserve.address);
        this.lendingIxs.push((0, instructions_1.repayObligationLiquidityWithNFTVoucherInstruction)(this.amount, this.userTokenAccountAddress, new web3_js_1.PublicKey(this.reserve.liquidityAddress), new web3_js_1.PublicKey(this.reserve.address), this.obligationAddress, new web3_js_1.PublicKey(this.lendingMarket.address), this.publicKey, this.programId, this.nftVoucherAddress, this.nftAtaAddress, new web3_js_1.PublicKey(reserveInfo.pythOracle), oracleProduct.key));
    }
    async addSupportIxs(action) {
        if (["withdraw", "borrow"].includes(action)) {
            await this.addRefreshIxs();
        }
        if (!["mint", "redeem"].includes(action)) {
            await this.addObligationIxs();
        }
        await this.addAtaIxs(action);
    }
    async addRefreshIxs() {
        // Union of addresses
        const allReserveAddresses = [
            ...Array.from([
                ...this.depositReserves.map((e) => e.toBase58()),
                ...this.borrowReserves.map((e) => e.toBase58()),
                this.reserve.address,
            ]),
        ];
        const uniqueAllReserveAddresses = [...new Set(allReserveAddresses)];
        const pda = (0, constants_renec_1.getRenecPDAOracle)();
        uniqueAllReserveAddresses.forEach((reserveAddress) => {
            const reserveInfo = this.lendingMarket.reserves.find((reserve) => reserve.address === reserveAddress);
            if (!reserveInfo) {
                throw new Error(`Could not find asset ${reserveAddress} in reserves`);
            }
            const symbol = reserveInfo.liquidityToken.symbol;
            let pair;
            if (symbol === constants_common_1.RE_USD) {
                pair = REUSD_PRICE;
            }
            else if (symbol === constants_common_1.RE_VND) {
                pair = REVND_PRICE;
            }
            else if (symbol === constants_common_1.RE_NGN) {
                pair = RENGN_PRICE;
            }
            else {
                pair = PRICE_PAIRS.find((pair) => pair.baseSymbol === reserveInfo.liquidityToken.symbol);
            }
            const oracleProduct = pda.product(new web3_js_1.PublicKey(pair.quoteToken), new web3_js_1.PublicKey(pair.baseToken));
            const refreshReserveIx = (0, instructions_1.refreshReserveRenecInstruction)(new web3_js_1.PublicKey(reserveAddress), this.programId, new web3_js_1.PublicKey(reserveInfo.pythOracle), oracleProduct.key, new web3_js_1.PublicKey(reserveInfo.switchboardOracle));
            this.setupIxs.push(refreshReserveIx);
        });
        const refreshObligationIx = (0, instructions_1.refreshObligationInstruction)(this.obligationAddress, this.depositReserves, this.borrowReserves, this.programId);
        this.setupIxs.push(refreshObligationIx);
    }
    async addObligationIxs() {
        if (!this.obligationAccountInfo) {
            const obligationAccountInfoRentExempt = await this.connection.getMinimumBalanceForRentExemption(obligation_1.OBLIGATION_SIZE);
            this.setupIxs.push(web3_js_1.SystemProgram.createAccountWithSeed({
                fromPubkey: this.publicKey,
                newAccountPubkey: this.obligationAddress,
                basePubkey: this.publicKey,
                seed: this.seed,
                lamports: obligationAccountInfoRentExempt,
                space: obligation_1.OBLIGATION_SIZE,
                programId: this.programId,
            }));
            const initObligationIx = (0, instructions_1.initObligationInstruction)(this.obligationAddress, new web3_js_1.PublicKey(this.lendingMarket.address), this.publicKey, this.programId);
            this.setupIxs.push(initObligationIx);
        }
    }
    async addAtaIxs(action) {
        if (this.symbol === "RENEC") {
            await this.updateWRENECAccount(action);
        }
        if ((action === "withdraw" || action === "borrow" || action === "redeem") && this.symbol !== "RENEC") {
            const userTokenAccountInfo = await this.connection.getAccountInfo(this.userTokenAccountAddress);
            if (!userTokenAccountInfo) {
                const createUserTokenAccountIx = (0, utils_1.createAssociatedTokenAccountInstruction)(this.publicKey, this.userTokenAccountAddress, this.publicKey, new web3_js_1.PublicKey(this.reserve.liquidityToken.mint));
                if (this.positions === constants_common_1.POSITION_LIMIT && this.hostAta) {
                    this.preTxnIxs.push(createUserTokenAccountIx);
                }
                else {
                    this.setupIxs.push(createUserTokenAccountIx);
                }
            }
        }
        if (action === "withdraw" || action === "mint" || action === "deposit") {
            const userCollateralAccountInfo = await this.connection.getAccountInfo(this.userCollateralAccountAddress);
            if (!userCollateralAccountInfo) {
                const createUserCollateralAccountIx = (0, utils_1.createAssociatedTokenAccountInstruction)(this.publicKey, this.userCollateralAccountAddress, this.publicKey, new web3_js_1.PublicKey(this.reserve.collateralMintAddress));
                if (this.positions === constants_common_1.POSITION_LIMIT && this.symbol === "RENEC") {
                    this.preTxnIxs.push(createUserCollateralAccountIx);
                }
                else {
                    this.setupIxs.push(createUserCollateralAccountIx);
                }
            }
        }
    }
    async updateWRENECAccount(action) {
        const preIxs = [];
        const postIxs = [];
        let safeRepay = new bn_js_1.default(this.amount);
        if (this.nftVoucherAddress && this.nftInterestAmount && this.nftInterestAmount.gtn(0))
            safeRepay = safeRepay.add(this.nftInterestAmount);
        if (this.obligationAccountInfo && action === "repay" && this.amount.eq(new bn_js_1.default(constants_1.U64_MAX))) {
            const buffer = await this.connection.getAccountInfo(new web3_js_1.PublicKey(this.reserve.address), "processed");
            if (!buffer) {
                throw Error(`Unable to fetch reserve data for ${this.reserve.liquidityToken.name}`);
            }
            const parsedData = (0, reserve_1.parseReserve)(new web3_js_1.PublicKey(this.reserve.address), buffer)?.info;
            if (!parsedData) {
                throw Error(`Unable to parse data of reserve ${this.reserve.liquidityToken.name}`);
            }
            const borrow = this.obligationAccountInfo.borrows.find((borrow) => borrow.borrowReserve.toBase58() === this.reserve.address);
            if (!borrow) {
                throw Error(`Unable to find obligation borrow to repay for ${this.obligationAccountInfo.owner.toBase58()}`);
            }
            safeRepay = new bn_js_1.default(Math.floor(new bignumber_js_1.default(borrow.borrowedAmountWads.toString())
                .multipliedBy(parsedData.liquidity.cumulativeBorrowRateWads.toString())
                .dividedBy(borrow.cumulativeBorrowRateWads.toString())
                .dividedBy(constants_1.WAD)
                .plus(RENEC_PADDING_FOR_INTEREST)
                .toNumber()).toString());
        }
        const userWRENECAccountInfo = await this.connection.getAccountInfo(this.userTokenAccountAddress);
        const rentExempt = await spl_token_1.Token.getMinBalanceRentForExemptAccount(this.connection);
        const sendAction = action === "deposit" || action === "repay" || action === "mint";
        const transferLamportsIx = web3_js_1.SystemProgram.transfer({
            fromPubkey: this.publicKey,
            toPubkey: this.userTokenAccountAddress,
            lamports: (userWRENECAccountInfo ? 0 : rentExempt) + (sendAction ? parseInt(safeRepay.toString(), 10) : 0),
        });
        preIxs.push(transferLamportsIx);
        const closeWRENECAccountIx = spl_token_1.Token.createCloseAccountInstruction(spl_token_1.TOKEN_PROGRAM_ID, this.userTokenAccountAddress, this.publicKey, this.publicKey, []);
        if (userWRENECAccountInfo) {
            const syncIx = (0, instructions_1.syncNative)(this.userTokenAccountAddress);
            if (sendAction) {
                preIxs.push(syncIx);
            }
            else {
                postIxs.push(closeWRENECAccountIx);
            }
        }
        else {
            const createUserWRENECAccountIx = (0, utils_1.createAssociatedTokenAccountInstruction)(this.publicKey, this.userTokenAccountAddress, this.publicKey, spl_token_1.NATIVE_MINT);
            preIxs.push(createUserWRENECAccountIx);
            postIxs.push(closeWRENECAccountIx);
        }
        if (this.positions && this.positions >= constants_common_1.POSITION_LIMIT) {
            this.preTxnIxs.push(...preIxs);
            this.postTxnIxs.push(...postIxs);
        }
        else {
            this.setupIxs.push(...preIxs);
            this.cleanupIxs.push(...postIxs);
        }
    }
    /**
     * Add the supply to earn instruction to the deposit instruction.
     */
    addSteInstructionWhenDepositing() {
        if (!this.steInstruction) {
            return;
        }
        // With RENEC case, we need to submit the steInstruction after the deposit instruction because in the first time when there is no obligation, the tx may fail with error Transaction too large: 1234 > 1232
        if (this.symbol === constants_common_1.RENEC) {
            this.postTxnIxs.push(this.steInstruction);
        }
        else {
            this.lendingIxs.push(this.steInstruction);
        }
    }
}
exports.RelendAction = RelendAction;
