import { ActionType } from "@/core";
import { Connection, PublicKey, TransactionInstruction } from "@solana/web3.js";
import { Obligation } from "@/state/obligation";
import BN from "bn.js";
export type RewardInfoType = {
    rewardRate: string;
    rewardMint?: string;
    rewardSymbol: string;
    price: number;
};
export type RewardsDataType = {
    [key: string]: {
        supply: Array<RewardInfoType>;
        borrow: Array<RewardInfoType>;
    };
};
export type RewardResponseType = {
    supply: RewardStatType;
    borrow: RewardStatType;
};
export type RewardStatType = {
    rewardsPerShare: string;
    totalBalance: string;
    lastSlot: number;
    rewardRates: Array<{
        beginningSlot: number;
        rewardRate: string;
        name?: string;
    }>;
} | null;
export type ExternalRewardStatType = RewardStatType & {
    rewardMint: string;
    rewardSymbol: string;
    reserveID: string;
    side: "supply" | "borrow";
};
export type ConfigType = Array<MarketConfigType>;
export type ReserveConfigType = {
    liquidityToken: {
        coingeckoID: string;
        decimals: number;
        logo: string;
        mint: string;
        name: string;
        symbol: string;
        volume24h: number;
    };
    pythOracle: string;
    switchboardOracle: string;
    address: string;
    collateralMintAddress: string;
    collateralSupplyAddress: string;
    liquidityAddress: string;
    liquidityFeeReceiverAddress: string;
    userSupplyCap: number;
    userBorrowCap: number;
};
export type MarketConfigType = {
    name: string;
    isPrimary: boolean;
    description: string;
    creator: string;
    address: string;
    hidden: boolean;
    authorityAddress: string;
    reserves: Array<ReserveConfigType>;
};
export type ReserveDataType = {
    optimalUtilizationRate: number;
    loanToValueRatio: number;
    liquidationBonus: number;
    liquidationThreshold: number;
    minBorrowRate: number;
    optimalBorrowRate: number;
    maxBorrowRate: number;
    borrowFeePercentage: number;
    flashLoanFeePercentage: number;
    hostFeePercentage: number;
    depositLimit: BN;
    reserveBorrowLimit: BN;
    name: string;
    symbol: string;
    decimals: number;
    mintAddress: string;
    totalDepositsWads: BN;
    totalBorrowsWads: BN;
    totalLiquidityWads: BN;
    supplyInterestAPY: number;
    borrowInterestAPY: number;
    assetPriceUSD: number;
    protocolTakeRate: number;
    userDepositLimit?: number;
    cumulativeBorrowRateWads: BN;
    cTokenExchangeRate: number;
};
export interface SupplyToEarnApy {
    account: SupplyToEarnAccount;
    publicKey: string;
}
export interface SupplyToEarnAccount {
    apy: number;
    rewardToken: string;
    tokenDecimals: number;
    reserve: string;
    chainId: number;
    address: string;
    name: string;
    symbol: string;
    decimals: number;
    logoURI: string;
    tags: string[];
    extensions: Extensions;
    startTime: number;
    endTime: number;
}
export interface SupplyToEarnReward {
    totalRewardNumber: string;
    accumulatedRewardAmount: string;
    lastSupply: string;
    apy: number;
    supplyAmount: string;
    chainId: number;
    address: string;
    name: string;
    symbol: string;
    decimals: number;
    logoURI: string;
    tags: string[];
    extensions: Extensions;
    currentTime: string;
    reserve: string;
    startTime: number;
    endTime: number;
}
export declare enum BlockchainNetwork {
    RENEC = "RENEC",
    SOLANA = "SOLANA"
}
export interface Extensions {
    [key: string]: string;
}
export declare enum ActionEnum {
    DEPOSIT = "Deposit",
    BORROW = "Borrow",
    WITHDRAW = "Withdraw",
    REPAY = "Repay"
}
export type BuildTxnsParams = {
    action: ActionEnum;
    network: BlockchainNetwork;
    connection: Connection;
    amount: string;
    symbol: string;
    publicKey: PublicKey;
    supplyToEarnApys?: SupplyToEarnApy[];
    supplyToEarnRewards?: SupplyToEarnReward[];
    lendingMarketAddress?: string;
    referralAccount?: PublicKey;
    nftVoucherAddress?: PublicKey;
    nftInterestAmount?: string;
};
export type ActionFunctionParams = {
    network: BlockchainNetwork;
    connection: Connection;
    amount: string | BN;
    symbol: string;
    publicKey: PublicKey;
    supplyToEarnApys?: SupplyToEarnApy[];
    supplyToEarnRewards?: SupplyToEarnReward[];
    lendingMarketAddress?: PublicKey;
    hostAddress?: PublicKey;
    nftVoucherAddress?: PublicKey;
    nftInterestAmount?: string;
};
export type ChainActionParams = ActionFunctionParams & {
    action: ActionType;
};
export type BlockchainActionOptions = {
    network: BlockchainNetwork;
    programId: PublicKey;
    connection: Connection;
    reserve: ReserveConfigType;
    lendingMarket: MarketConfigType;
    publicKey: PublicKey;
    obligationAddress: PublicKey;
    obligationAccountInfo: Obligation | null;
    userTokenAccountAddress: PublicKey;
    userCollateralAccountAddress: PublicKey;
    seed: string;
    symbol: string;
    positions: number;
    amount: string | BN;
    depositReserves: Array<PublicKey>;
    borrowReserves: Array<PublicKey>;
    hostAta?: PublicKey;
    setupIxs: TransactionInstruction[];
    supplyToEarnApys?: SupplyToEarnApy[];
    supplyToEarnRewards?: SupplyToEarnReward[];
    nftVoucherAddress?: PublicKey;
    nftAtaAddress?: PublicKey;
    nftInterestAmount?: string | BN;
};
