"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.calculateBorrowAPR = exports.calculateUtilizationRatio = exports.calculateSupplyAPR = exports.calculateBorrowAPY = exports.calculateSupplyAPY = exports.calculateNewRatesAfterRefreshReserve = void 0;
const classes_1 = require("../../classes/index.js");
const utils_1 = require("../../classes/utils.js");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const constants_common_1 = require("../constants.common.js");
const calculateNewRatesAfterRefreshReserve = (reserve, currentSlot) => {
    const slotElapsed = currentSlot - (reserve.lastUpdateSlot.toNumber() ?? 0);
    const reserveCummulative = reserve.cumulativeBorrowRate;
    const currentBorrowRate = (0, exports.calculateBorrowAPR)(reserve);
    const compoundedInterestRateWhenRefreshReserve = (0, utils_1.calculateCompoundedInterestRate)(currentBorrowRate, slotElapsed);
    const reserveBorrowedAmount = new bignumber_js_1.default(reserve.totalBorrowWad.toString()).div(classes_1.WAD);
    const netNewDebt = reserveBorrowedAmount
        .multipliedBy(compoundedInterestRateWhenRefreshReserve)
        .minus(reserveBorrowedAmount);
    const accumulatedProtocolFees = new bignumber_js_1.default(reserve.accumulatedProtocolFeesWad.toString()).div(classes_1.WAD);
    const newBorrowedAmountAfterRefresh = reserveBorrowedAmount.plus(netNewDebt);
    const newAccumulatedProtocolTakeFee = netNewDebt
        .multipliedBy(reserve.interestRateSpread)
        .plus(accumulatedProtocolFees);
    const availableAmount = new bignumber_js_1.default(reserve.availableAmount.toString());
    const newTotalLiquidity = availableAmount.plus(newBorrowedAmountAfterRefresh).minus(newAccumulatedProtocolTakeFee);
    const mintTotalSupply = reserve.mintTotalSupplyWad;
    const newCTokenExchangeRate = new bignumber_js_1.default(newTotalLiquidity.div(new bignumber_js_1.default(mintTotalSupply.toString())));
    const newCummulativeBorrowRate = reserveCummulative.multipliedBy(compoundedInterestRateWhenRefreshReserve);
    return {
        newAccumulatedProtocolTakeFee,
        newCummulativeBorrowRate,
        newBorrowedAmountAfterRefresh,
        newCTokenExchangeRate,
    };
};
exports.calculateNewRatesAfterRefreshReserve = calculateNewRatesAfterRefreshReserve;
const calculateSupplyAPY = (reserve) => {
    const apr = (0, exports.calculateSupplyAPR)(reserve);
    const apy = new bignumber_js_1.default(1).plus(new bignumber_js_1.default(apr).dividedBy(constants_common_1.SLOTS_PER_YEAR)).toNumber() ** constants_common_1.SLOTS_PER_YEAR - 1;
    return apy;
};
exports.calculateSupplyAPY = calculateSupplyAPY;
const calculateBorrowAPY = (reserve) => {
    const apr = (0, exports.calculateBorrowAPR)(reserve);
    const apy = new bignumber_js_1.default(1).plus(new bignumber_js_1.default(apr).dividedBy(constants_common_1.SLOTS_PER_YEAR)).toNumber() ** constants_common_1.SLOTS_PER_YEAR - 1;
    return apy;
};
exports.calculateBorrowAPY = calculateBorrowAPY;
const calculateSupplyAPR = (reserve) => {
    const currentUtilization = (0, exports.calculateUtilizationRatio)(reserve);
    const borrowAPY = (0, exports.calculateBorrowAPR)(reserve);
    return currentUtilization.multipliedBy(borrowAPY);
};
exports.calculateSupplyAPR = calculateSupplyAPR;
const calculateUtilizationRatio = (reserve) => {
    const totalBorrowsWads = new bignumber_js_1.default(reserve.totalBorrowWad.toString()).div(classes_1.WAD);
    const currentUtilization = totalBorrowsWads.dividedBy(totalBorrowsWads.plus(reserve.availableAmountWad.toString()));
    return currentUtilization;
};
exports.calculateUtilizationRatio = calculateUtilizationRatio;
const calculateBorrowAPR = (reserve) => {
    const currentUtilization = (0, exports.calculateUtilizationRatio)(reserve);
    const optimalUtilization = reserve.targetUtilization;
    let borrowAPR;
    if (optimalUtilization === 1.0 || currentUtilization.isLessThan(optimalUtilization)) {
        const normalizedFactor = currentUtilization.div(optimalUtilization);
        const optimalBorrowRate = reserve.targetBorrowApr;
        const minBorrowRate = reserve.minBorrowApr;
        borrowAPR = normalizedFactor.multipliedBy(optimalBorrowRate - minBorrowRate).plus(minBorrowRate);
    }
    else {
        const normalizedFactor = currentUtilization.minus(optimalUtilization).div(1 - optimalUtilization);
        const optimalBorrowRate = reserve.targetBorrowApr;
        const maxBorrowRate = reserve.maxBorrowApr;
        borrowAPR = normalizedFactor.multipliedBy(maxBorrowRate - optimalBorrowRate).plus(optimalBorrowRate);
    }
    return borrowAPR;
};
exports.calculateBorrowAPR = calculateBorrowAPR;
