"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchTokensInfoRenec = exports.fetchTokensInfoSolana = exports.fetchTokensInfoSolanaMetaplex = exports.fetchTokensInfo = void 0;
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const classes_1 = require("../../classes/index.js");
const mpl_token_metadata_1 = require("@metaplex-foundation/mpl-token-metadata");
const umi_1 = require("@metaplex-foundation/umi");
const umi_bundle_defaults_1 = require("@metaplex-foundation/umi-bundle-defaults");
const spl_token_registry_1 = require("@solana/spl-token-registry");
const axios_1 = __importDefault(require("axios"));
const custom_utl_sdk_1 = __importDefault(require("../../utils/custom-utl-sdk/index.js"));
const constants_common_1 = require("../constants.common.js");
const constants_helper_1 = require("../constants.helper.js");
const fetchTokensInfo = async (network, mints, debug) => {
    if (debug)
        console.log("getTokensInfo");
    switch (network) {
        case classes_1.BlockchainNetwork.SOLANA:
            return (0, exports.fetchTokensInfoSolana)(network, mints);
        case classes_1.BlockchainNetwork.RENEC:
            return (0, exports.fetchTokensInfoRenec)(mints);
        default:
            throw new Error("Unsupported blockchain for token info");
    }
};
exports.fetchTokensInfo = fetchTokensInfo;
const fetchTokensInfoSolanaMetaplex = async (network, liquidityPubkeys) => {
    const umi = (0, umi_bundle_defaults_1.createUmi)(constants_helper_1.ConstantsHelper.getRPCEndpointUrl(network)).use((0, mpl_token_metadata_1.mplTokenMetadata)());
    const publicKeys = liquidityPubkeys.map((pubkey) => (0, umi_1.publicKey)(pubkey));
    const assets = await (0, mpl_token_metadata_1.fetchAllDigitalAsset)(umi, publicKeys);
    const promises = [];
    for (const asset of assets) {
        promises.push(axios_1.default.get(asset.metadata.uri));
    }
    const fetchAssetsMetadata = await Promise.all(promises);
    return Object.fromEntries(assets.map((asset) => [
        asset.mint.publicKey.toString(),
        {
            name: asset.metadata.name,
            symbol: asset.metadata.symbol,
            logoUri: fetchAssetsMetadata.find((response) => response.data?.symbol === asset.metadata.symbol)?.data.image,
            decimals: asset.mint.decimals,
        },
    ]));
};
exports.fetchTokensInfoSolanaMetaplex = fetchTokensInfoSolanaMetaplex;
const fetchTokensInfoSolana = async (network, mints) => {
    const marketInfo = constants_helper_1.ConstantsHelper.getConfig(network)[0];
    if (constants_common_1.IS_MAINNET) {
        const provider = await new spl_token_registry_1.TokenListProvider().resolve();
        const tokenList = provider.filterByChainId(spl_token_registry_1.ENV.MainnetBeta).getList();
        const tokens = tokenList.filter((token) => mints.find((mint) => mint === token.address) !== undefined);
        // Collect all undefined mints
        const undefinedMints = mints.filter((mint) => !tokens.some((token) => token.address === mint));
        let metaplexMints;
        if (undefinedMints.length > 0) {
            metaplexMints = await (0, exports.fetchTokensInfoSolanaMetaplex)(network, undefinedMints);
        }
        const splTokenMints = Object.fromEntries(tokens
            .map((token) => token
            ? [
                token.address,
                {
                    name: token.name ?? "",
                    symbol: token.symbol,
                    logoUri: token.logoURI,
                    decimals: token.decimals ?? 0,
                },
            ]
            : [])
            .filter((x) => x.length));
        return {
            ...splTokenMints,
            ...metaplexMints,
        };
    }
    const tokens = marketInfo.reserves.map((r) => r.liquidityToken);
    return Object.fromEntries(tokens
        .map((token) => token
        ? [
            token.mint,
            {
                name: token.name ?? "",
                symbol: token.symbol,
                logoUri: token.logo,
                decimals: token.decimals ?? 0,
            },
        ]
        : [])
        .filter((x) => x.length));
};
exports.fetchTokensInfoSolana = fetchTokensInfoSolana;
const fetchTokensInfoRenec = async (mints) => {
    const utl = new custom_utl_sdk_1.default();
    const tokens = await utl.fetchMints("https://raw.githubusercontent.com/renec-chain/rpl-token-registry/master/src/tokens/renec.tokenlist.json", mints.map((mint) => new web3_js_1.PublicKey(mint)));
    let output = Object.fromEntries(tokens
        .map((token) => token
        ? [
            token.address,
            {
                name: token.name ?? "",
                symbol: token.symbol,
                logoUri: token.logoURI,
                decimals: token.decimals ?? 0,
            },
        ]
        : [])
        .filter((x) => x.length));
    if (mints.includes(spl_token_1.NATIVE_MINT.toBase58())) {
        output = {
            ...output,
            [spl_token_1.NATIVE_MINT.toBase58()]: {
                name: "RENEC",
                symbol: "RENEC",
                logoUri: "https://easytokenbucket.s3.amazonaws.com/nsbr7a24f6ox3xh40obw8jaaxjzt",
                decimals: 9,
            },
        };
    }
    if (mints.includes("USDbgSB1DPBCEDt15ppgTZZYiQokMHcvkX6JQRfVNJY")) {
        output = {
            ...output,
            USDbgSB1DPBCEDt15ppgTZZYiQokMHcvkX6JQRfVNJY: {
                name: "USDT (reUSD)",
                symbol: constants_common_1.RE_USD,
                logoUri: "https://raw.githubusercontent.com/renec-chain/rpl-token-registry/master/assets/mainnet/4Q89182juiadeFgGw3fupnrwnnDmBhf7e7fHWxnUP3S3/logo.png",
                decimals: 9,
            },
        };
    }
    if (mints.includes("4Q89182juiadeFgGw3fupnrwnnDmBhf7e7fHWxnUP3S3")) {
        output = {
            ...output,
            "4Q89182juiadeFgGw3fupnrwnnDmBhf7e7fHWxnUP3S3": {
                name: "USDT (reUSD)",
                symbol: constants_common_1.RE_USD,
                logoUri: "https://raw.githubusercontent.com/renec-chain/rpl-token-registry/master/assets/mainnet/4Q89182juiadeFgGw3fupnrwnnDmBhf7e7fHWxnUP3S3/logo.png",
                decimals: 9,
            },
        };
    }
    if (mints.includes("Dtu6zyXkSAVxzFFLkj85ZsEm5woYuMbcnLpfnb3LRksX")) {
        output = {
            ...output,
            "Dtu6zyXkSAVxzFFLkj85ZsEm5woYuMbcnLpfnb3LRksX": {
                name: "BNB (reBNB)",
                symbol: "reBNB",
                logoUri: "https://easytokenbucket.s3.amazonaws.com/7gugsoz5j6d2tnu9omg8ymjwu2qx",
                decimals: 9,
            },
        };
    }
    if (mints.includes("Emqxh2Z6dnFEvBka77zqS1xq6Zkoxa8MduBHswqYxsez")) {
        output = {
            ...output,
            "Emqxh2Z6dnFEvBka77zqS1xq6Zkoxa8MduBHswqYxsez": {
                name: "VND (reVND)",
                symbol: "reVND",
                logoUri: "https://easytokenbucket.s3.amazonaws.com/49eub7281ftwec6s2ehecsycmvby",
                decimals: 0,
            },
        };
    }
    if (mints.includes("Dtu6zyXkSAVxzFFLkj85ZsEm5woYuMbcnLpfnb3LRksX")) {
        output = {
            ...output,
            "Dtu6zyXkSAVxzFFLkj85ZsEm5woYuMbcnLpfnb3LRksX": {
                name: "BNB (reBNB)",
                symbol: "reBNB",
                logoUri: "https://easytokenbucket.s3.amazonaws.com/7gugsoz5j6d2tnu9omg8ymjwu2qx",
                decimals: 9,
            },
        };
    }
    if (mints.includes("Emqxh2Z6dnFEvBka77zqS1xq6Zkoxa8MduBHswqYxsez")) {
        output = {
            ...output,
            "Emqxh2Z6dnFEvBka77zqS1xq6Zkoxa8MduBHswqYxsez": {
                name: "VND (reVND)",
                symbol: "reVND",
                logoUri: "https://easytokenbucket.s3.amazonaws.com/49eub7281ftwec6s2ehecsycmvby",
                decimals: 0,
            },
        };
    }
    // reVND testnet
    if (mints.includes("8NoZYiawFCNhGUSDoLAy3xr2DMCKnnn96vbTvjw8Ykzs")) {
        output = {
            ...output,
            "8NoZYiawFCNhGUSDoLAy3xr2DMCKnnn96vbTvjw8Ykzs": {
                name: constants_common_1.RE_VND,
                symbol: constants_common_1.RE_VND,
                logoUri: "https://easytokenbucket.s3.amazonaws.com/y5ugt406h4x6nrat5u4s1cbyyaw9",
                decimals: 0,
            },
        };
    }
    return output;
};
exports.fetchTokensInfoRenec = fetchTokensInfoRenec;
