"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const classes_1 = require("../../classes/index.js");
const state_1 = require("../../state/index.js");
const custom_oracle_1 = require("../../utils/custom-oracle/index.js");
const web3_js_1 = require("@solana/web3.js");
const bn_js_1 = require("bn.js");
const constants_common_1 = require("../constants.common.js");
const constants_renec_1 = require("../constants.renec.js");
const constants_solana_1 = require("../constants.solana.js");
const prices_1 = require("./prices.js");
const TEST_BLOCKCHAIN_NETWORK = classes_1.BlockchainNetwork.RENEC;
jest.mock("@/utils/custom-oracle");
jest.mock("../constants.renec", () => {
    const originalModule = jest.requireActual("../constants.renec");
    return {
        __esModule: true,
        ...originalModule,
        getPricePair: jest.fn().mockReturnValue(originalModule.PRICE_PAIRS_MAINNET),
        getReUSDPricePair: jest.fn().mockReturnValue(originalModule.REUSD_PRICE_MAINNET),
        getPDA: jest.fn().mockReturnValue(originalModule.PDA_ORACLE_MAINNET),
        getRelendInfo: jest.fn().mockReturnValue(originalModule.RELEND_INFO_MAINNET),
    };
});
const RESERVE_RE_BTC_MAINNET = "AtiGYDhwF7WgxCCkoBrBZExGgvkMdZvjF8RbeiFz5JYn";
const RESERVE_RE_VND_MAINNET = "HrViAzmrXZVtvhP6JPSyvcw1EqWNWfUwdVAPfyMwU3rU";
describe("fetchPrices", () => {
    const OLD_ENV = process.env;
    // Common setup for the tests
    let mockMainnetConnection;
    let mockReserveParsed;
    const MOCK_PARSE_PRICE_RETURN_VALUE = {
        version: 1,
        status: { online: {} },
        productAccount: new web3_js_1.PublicKey("4kn3JeaXhBbbwKF3kaYubiWHVCxkJdJap6zsm54CS4YQ"),
        prevPrice: new bn_js_1.BN("43233463396"),
        price: new bn_js_1.BN("43235682627"),
        numPublishers: 1,
        prevTimestamp: new bn_js_1.BN("1705420804"),
        timestamp: new bn_js_1.BN("1705420824"),
        bump: [255],
    };
    const MOCK_PARSE_PRODUCT_RETURN_VALUE = {
        version: 1,
        status: { online: {} },
        assetType: { forex: {} },
        quoteCurrency: "reUSD",
        quoteMint: new web3_js_1.PublicKey("USDbgSB1DPBCEDt15ppgTZZYiQokMHcvkX6JQRfVNJY"),
        baseCurrency: "reBTC",
        baseMint: new web3_js_1.PublicKey("BTCxWeEcT5sYsys7ncnr2bU1Mzbn1aXXqJDv43X22cDy"),
        priceAccount: new web3_js_1.PublicKey("EWMPWwzt5wFWmtsYhEvGxZ8EvEsPp9Fm5rfQBwoXR3Nz"),
        expo: -6,
        maxPrice: new bn_js_1.BN("174876e800", 16),
        minPrice: new bn_js_1.BN("2540be400", 16),
        windowSize: new bn_js_1.BN("14", 16),
        controller: new web3_js_1.PublicKey("B3cWF2ZF5ZJ1bjZfxmKUdpUGR7JDSvJens2bZsmWBnnk"),
        bump: [255],
    };
    beforeEach(async () => {
        // Reset mocks and set default return values
        jest.mocked(custom_oracle_1.parsePriceOracle).mockResolvedValue(MOCK_PARSE_PRICE_RETURN_VALUE);
        jest.mocked(custom_oracle_1.parseProductOracle).mockResolvedValue(MOCK_PARSE_PRODUCT_RETURN_VALUE);
        // Set up mock data
        mockMainnetConnection = new web3_js_1.Connection(constants_renec_1.RENEC_MAINNET_URL);
        const filters = [{ dataSize: 619 }];
        const rawReserves = await mockMainnetConnection.getProgramAccounts(classes_1.RELEND_PRODUCTION_PROGRAM_ID, {
            commitment: mockMainnetConnection.commitment,
            filters,
            encoding: "base64",
        });
        const parsedReserves = rawReserves
            .map((reserve, index) => (reserve ? (0, state_1.parseReserve)(rawReserves[index].pubkey, reserve.account) : null))
            .filter(Boolean);
        mockReserveParsed = parsedReserves;
    });
    afterAll(() => {
        process.env = OLD_ENV;
    });
    it("should return correct prices", async () => {
        const prices = await (0, prices_1.fetchPrices)(TEST_BLOCKCHAIN_NETWORK, mockReserveParsed, mockMainnetConnection);
        expect(prices).toHaveProperty(RESERVE_RE_BTC_MAINNET);
        expect(prices[RESERVE_RE_BTC_MAINNET]).toEqual({ spotPrice: 43235.682626999995, emaPrice: 43235.682626999995 });
    });
    it("should return default price if price is null/undefined/0 for normal token and fiat token", async () => {
        jest.mocked(custom_oracle_1.parsePriceOracle).mockResolvedValue(null);
        const prices = await (0, prices_1.fetchPrices)(TEST_BLOCKCHAIN_NETWORK, mockReserveParsed, mockMainnetConnection);
        console.log(prices);
        expect(prices).toHaveProperty(RESERVE_RE_BTC_MAINNET);
        expect(prices[RESERVE_RE_BTC_MAINNET]).toEqual({
            spotPrice: prices_1.DEFAULT_PRICES[constants_common_1.RE_BTC],
            emaPrice: prices_1.DEFAULT_PRICES[constants_common_1.RE_BTC],
        });
        expect(prices).toHaveProperty(RESERVE_RE_VND_MAINNET);
        expect(prices[RESERVE_RE_VND_MAINNET]).toEqual({
            spotPrice: prices_1.DEFAULT_PRICES[constants_common_1.RE_VND],
            emaPrice: prices_1.DEFAULT_PRICES[constants_common_1.RE_VND],
        });
    });
});
const solanaReserveLog = {
    version: 1,
    lastUpdate: {
        slot: 73708305,
        stale: 1,
    },
    lendingMarket: "S3nJfDVgc5pe5vjPr8CauSxPwKhAJJwfyHA3pPzrD3A",
    liquidity: {
        mintPubkey: "7Zi4cotUxpS11wNE1TYjotv2RuVdHTdrm5S5VcpvNXfD",
        mintDecimals: 6,
        supplyPubkey: "9rMUcGVN2kFZS7TAsa5Psg5zXBCuAJQicDtcSYSZPrqP",
        oracleOption: undefined,
        pythOracle: "EdVCmQ9FSPcVe5YySXDPCRmc8aDQLKJ9xvYBMZPie1Vw",
        switchboardOracle: "QJc2HgGhdtW4e7zjvLB1TGRuwEpTre2agU5Lap2UqYz",
        availableAmount: "250000000",
        borrowedAmountWads: "0",
        cumulativeBorrowRateWads: "1000000000000000000",
        marketPrice: "3128881000000000000000",
        accumulatedProtocolFeesWads: "0",
        smoothedMarketPrice: undefined,
    },
    collateral: {
        mintPubkey: "J8Hr8CP8ZV2eKa8M7PAZorsXy7ecs6tM7GW5jkpR7NcX",
        mintTotalSupply: "250000000",
        supplyPubkey: "89Dk8YQiHUoRC1KPG38N5EuHaNnzaQwqDT95zhGmQgeo",
    },
    config: {
        optimalUtilizationRate: 80,
        maxUtilizationRate: 80,
        loanToValueRatio: 75,
        liquidationBonus: 5,
        maxLiquidationBonus: 5,
        liquidationThreshold: 80,
        maxLiquidationThreshold: 80,
        minBorrowRate: 0,
        optimalBorrowRate: 8,
        maxBorrowRate: 100,
        superMaxBorrowRate: 100,
        fees: {
            borrowFeeWad: "100000000000000",
            flashLoanFeeWad: "3000000000000000",
            hostFeePercentage: 50,
        },
        depositLimit: "500000000",
        borrowLimit: "18446744073709551615",
        feeReceiver: "DM8EWDSZ4DLiGw1dgTQbZgEmb58EXo6jqbpV9u2CdQyu",
        protocolLiquidationFee: 0,
        protocolTakeRate: 0,
        addedBorrowWeightBPS: "0",
        borrowWeight: 1,
        reserveType: 0,
    },
    rateLimiter: {
        config: {
            maxOutflow: 0,
            windowDuration: 0,
        },
        previousQuantity: 0,
        windowStart: 0,
        currentQuantity: 0,
    },
};
const reserve = (0, state_1.constructReserveFromLog)(solanaReserveLog);
const MOCK_SOLANA_RESERVES = [
    {
        info: reserve,
        pubkey: new web3_js_1.PublicKey("BNT7dNxwm3y3bYjau5CRHEgHSbEzHQu5aKvZ65234Jse"),
    },
];
describe.only("fetchPriceSolana", () => {
    it.only("should return correct prices", async () => {
        const price = await (0, prices_1.fetchPricesSolana)(classes_1.BlockchainNetwork.SOLANA, MOCK_SOLANA_RESERVES, new web3_js_1.Connection(constants_solana_1.SOLANA_DEVNET_URL));
        console.log("🚀 ~ it.only ~ price:", price);
    });
});
